# imports
import os
import mlflow

from random import random

# define functions
def main():
    mlflow.log_param("example_param", "example")
    mlflow.log_metric("example_metric", random())
    os.system(f"echo 'hello world' > helloworld.txt")
    mlflow.log_artifact("helloworld.txt")


# run functions
if __name__ == "__main__":
    # run main function
    main()